#!/usr/bin/env python
#  A network visualization tool - (C) 2000 by Sami Kystil
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from Tkinter import *
from toolbar import *
from node import *
from mode import *
from SimpleDialog import *
import tkFileDialog
import string
import sys

class App:
	"""netmap v0.1.0 (C) 2000 by Sami Kystil"""
	def __init__(self,master):
		self.count=0
		self.objects={}
		self.selected=None
		self.gridsize=74
		self.toolbar=Toolbar(master)

		buttons=[
			("Create",self.button_add_node),
			("Move",self.button_arrange),
			("Link",self.button_link),
			("Unlink",self.button_unlink),
			("Delete",self.button_delete),
			("Properties",self.button_properties),
			("Operations",self.button_operations),
			]
			
		f=Frame(master)

		i=0
		for caption,func in buttons:
			Radiobutton(f,text=caption,command=func,value=i,indicatoron=0,selectcolor="#aaccee").pack(side=TOP,fill=X)
			i+=1

		Button(f,text="Save",command=self.button_save).pack(side=BOTTOM,fill=X)
		Button(f,text="Load",command=self.button_load).pack(side=BOTTOM,fill=X)
		Button(f,text="Clear",command=self.button_clear).pack(side=BOTTOM,fill=X)
		Button(f,text="Quit",command=self.button_quit).pack(side=BOTTOM,fill=X)
		f.pack(side=LEFT,fill=Y)

		sy=Scrollbar(master,orient=VERTICAL)
		sx=Scrollbar(master,orient=HORIZONTAL)
		sy.pack(side=RIGHT,fill=Y)
		sx.pack(side=BOTTOM,fill=X)
		self.canvas=Canvas(master,bg="white",width=400,height=300,yscrollcommand=sy.set,xscrollcommand=sx.set)
		self.canvas.pack(side=LEFT,fill=BOTH,expand=1)

		sy.config(command=self.canvas.yview)
		sx.config(command=self.canvas.xview)

		self.button_arrange()
		self.toolbar.set_status(self.__doc__)
	def update_selection(self):
		self.selected=[]
		for i in self.canvas.find_withtag("selected"):
			self.selected.append(self.objects[self.canvas.gettags(i)[1]])
	def update_view(self):
		self.canvas.config(scrollregion=self.canvas.bbox(ALL))
	def clear_selection(self):
		for i in self.canvas.find_withtag("selected"):
			o=self.objects[self.canvas.gettags(i)[1]]
			o.toggle(self.canvas,0)
		self.canvas.dtag("selected")
		self.update_selection()
	def add_node(self,x,y):	
		id=self.count
		n=Node(self.canvas,id=id,x=x,y=y,name="Node "+str(id))
		self.objects[n.tag[1]]=n
		self.count+=1
		self.update_view()
	def find_node(self,id):
		for o in self.objects.values():
			if o.id==id: return o
		return None
	def add_link(self,a,b,color):
		try:
			a.link(b,self.canvas,color)
			b.link(a,self.canvas,color)
		except LinkError:
			self.toolbar.set_status("Those nodes are already linked!")
	def clear(self):
		for o in self.objects.values():
			o.delete(self.canvas)
		self.objects={}
		self.count=0
		
	# buttons
	def button_arrange(self):
		self.mode=ModeHandler_Arrange(self)
		self.canvas.config(cursor="hand2")
	def button_add_node(self):
		self.mode=ModeHandler_Add(self)
		self.canvas.config(cursor="cross")
	def button_link(self):
		self.mode=ModeHandler_Link(self)
		self.canvas.config(cursor="")
	def button_unlink(self):
		self.mode=ModeHandler_Unlink(self)
		self.canvas.config(cursor="arrow")
	def button_delete(self):
		self.mode=ModeHandler_Delete(self)
		self.canvas.config(cursor="cross")
	def button_properties(self):			
		self.mode=ModeHandler_Properties(self)
		self.canvas.config(cursor="hand1")
	def button_operations(self):
		self.mode=ModeHandler_Operations(self)
		self.canvas.config(cursor="hand1")
	def button_load(self):
		# File format:
		#     normal node:     id, name, ip, x, y, type
		#     link:            *, from_id, to_id, color

		fn=tkFileDialog.askopenfilename(filetypes=[("network maps","*.nmap"),("all files", "*")])
		if not len(fn): return

		f=open(fn,"rb")
		lines=f.readlines()
		current=-1
		n=None
		self.clear()
		
		for l in lines:
			l=string.replace(l,"\010","")
			l=string.replace(l,"\012","")
			l=string.replace(l," ","")
			args=string.split(l,",")
			if args[0]!='*':
				try:
					n=Node(self.canvas,id=int(args[0]),name=args[1],ip=args[2],x=int(args[3]),y=int(args[4]),type=args[5])
					self.objects[n.tag[1]]=n
					self.count+=1
				except IndexError:
					pass

		for l in lines:
			l=string.replace(l,"\010","")
		        l=string.replace(l,"\012","")
			l=string.replace(l," ","")
			args=string.split(l,",")
			if args[0]=='*':
				frm=self.find_node(int(args[1]))
				to=self.find_node(int(args[2]))
				if frm==None or to==None:
					print "Invalid link from",args[1],"to",args[2]
					break
				frm.link(to,self.canvas,args[3])
				to.link(frm,self.canvas,args[3])
				
		f.close()
		self.toolbar.set_status("Loaded map from "+fn)
	def button_save(self):
		fn=tkFileDialog.asksaveasfilename(filetypes=[("network maps","*.nmap"),("all files", "*")])
		if not len(fn): return
		f=open(fn,"wb")

		for o in self.objects.values():
			s="%s, %s, %s, %d, %d, %s\n" % (o.id,o.name,o.ip,o.x,o.y,o.type)
			f.write(s)
			for to,h in o.links.items():
				if h!=None:
					f.write("*, %d, %d, %s\n" % (o.id,to.id,self.canvas.itemcget(h,"fill")))

		self.toolbar.set_status("Saved map to "+fn)
		f.close()

	def button_clear(self):
		d = SimpleDialog(root,
				 text="Really delete everything?",
				 buttons=["OK", "Cancel",],
				 default=1,
				 cancel=1,
				 title="Clear?")
		if d.go()==0: self.clear()

	def button_quit(self):
		d = SimpleDialog(root,
				 text="Really quit?\n(Have you saved?)",
				 buttons=["OK", "Cancel",],
				 default=1,
				 cancel=1,
				 title="Quit?")
		if d.go()==0: sys.exit()

	# toolbar
	def button_align(self):
		for i in self.canvas.find_withtag("selected"):
			o=self.objects[self.canvas.gettags(i)[1]]
			o.x=int(o.x+32)/self.gridsize*self.gridsize
			o.y=int(o.y+16)/self.gridsize*self.gridsize
			o.update(self.canvas)
			o.update_links(self.canvas)


if __name__ == "__main__":
	root=Tk()
	app=App(root)
	root.mainloop()

						    
