from Tkinter import *
from node import *
from toolbar import *
import sys
import string

class ModeHandler:
	"""A base class for all the different modes the editor can be in. Handles node selection and viewport panning by default."""
	def __init__(self,parent):
		self.parent=parent
		self.parent.canvas.bind("<Button-1>",self.canvas_click)
		self.parent.canvas.bind("<B1-Motion>",self.canvas_drag)
		self.parent.canvas.bind("<B1-ButtonRelease-1>",self.canvas_release)
		self.parent.canvas.bind("<Button-3>",self.canvas_click_r)
		self.parent.canvas.bind("<B3-Motion>",self.canvas_drag_r)
		self.parent.toolbar.clear()
		self.parent.clear_selection()
	def canvas_drag_r(self,event):
		self.parent.canvas.scan_dragto(int(event.x)/4,int(event.y)/4)
	def canvas_click_r(self,event):
		self.parent.canvas.scan_mark(int(event.x)/4,int(event.y)/4)
	def canvas_drag(self,event): pass
	def canvas_release(self,event): pass
	def canvas_click(self,event):
		c=self.parent.canvas
		if "node" in c.gettags(CURRENT):
			if not (event.state&4 or len(self.parent.selected)>1):
				self.parent.clear_selection()
			o=self.parent.objects[c.gettags(CURRENT)[1]]
			o.toggle(c,1)
			c.addtag_withtag("selected","current")

			for i in c.find_withtag("selected"):
				o=self.parent.objects[c.gettags(i)[1]]
				o.grab(c,c.canvasx(event.x),c.canvasy(event.y))
		else:
			self.parent.clear_selection()
		self.parent.update_selection()

class ModeHandler_Add(ModeHandler):
	"""A mode handler for adding new nodes."""
	def __init__(self,parent):
                ModeHandler.__init__(self,parent)
		self.parent.toolbar.clear()
	def canvas_click(self,event):
		self.parent.add_node(self.parent.canvas.canvasx(event.x),self.parent.canvas.canvasy(event.y))


class ModeHandler_Arrange(ModeHandler):
	"""A mode handler for arranging existing nodes."""
	def __init__(self,parent):
	        ModeHandler.__init__(self,parent)
		self.parent.toolbar.add(Button(self.parent.toolbar.frame,text="Align",command=self.parent.button_align))
	def canvas_drag_r(self,event):
		ModeHandler.canvas_drag_r(self,event)
	def canvas_drag(self,event):
		for o in self.parent.selected:
			o.drag(self.parent.canvas,self.parent.canvas.canvasx(event.x),self.parent.canvas.canvasy(event.y))
	def canvas_release(self,event):
		for o in self.parent.selected:
			o.drop(self.parent.canvas)
		self.parent.update_view()
	def canvas_click(self,event):
		ModeHandler.canvas_click(self,event)

class ModeHandler_Link(ModeHandler):
	"""A mode handler for creating links between existing nodes."""
	def __init__(self,parent):
		ModeHandler.__init__(self,parent)
		colors=["black","gray50","gray75","red","green3","blue","darkviolet","gold","darkcyan"]
		self.last=None
		self.col=StringVar()
		for c in colors:
			self.parent.toolbar.add(Radiobutton(self.parent.toolbar.frame,variable=self.col,bg=c,value=c,indicatoron=0,text="   ",selectcolor=c))
	def canvas_click(self,event):
		self.parent.clear_selection()		     
		ModeHandler.canvas_click(self,event)
		try:
			if (self.last!=None and self.last!=self.parent.selected[0]):
				self.parent.toolbar.set_status("Linked \'" + self.last.name + "\' with \'" + self.parent.selected[0].name + "\'.")
				self.parent.add_link(self.last,self.parent.selected[0],self.col.get())
			else:
				self.parent.toolbar.set_status("Link \'" + self.parent.selected[0].name + "\' with...")
				self.parent.canvas.config(cursor="arrow")
			self.last=self.parent.selected[0]
		except IndexError:
			self.last=None
			self.parent.toolbar.set_status("")
			self.parent.canvas.config(cursor="")

class ModeHandler_Unlink(ModeHandler):
	"""A mode handler for deleting existing links between nodes."""
	def __init__(self,parent):
                ModeHandler.__init__(self,parent)
		self.parent.toolbar.clear()
	def canvas_click(self,event):
		c=self.parent.canvas
		try:
			l=c.find_withtag("current")[0]
			if "link" in c.gettags(CURRENT):
	        		for id,o in self.parent.objects.items():
					o.unlink_by_handle(l,c)
		except IndexError:
			pass

class ModeHandler_Delete(ModeHandler):
	"""A mode handler for deleting existing nodes."""
	def __init__(self,parent):
                ModeHandler.__init__(self,parent)
		self.parent.toolbar.clear()
	def canvas_click(self,event):
		ModeHandler.canvas_click(self,event)
		try:
			self.parent.selected[0].delete(self.parent.canvas)
		except IndexError:
			pass
		self.parent.clear_selection()

class ModeHandler_Properties(ModeHandler):
	"""A mode handler for changing the properties of links and modes."""
	def __init__(self,parent):
		ModeHandler.__init__(self,parent)
                self.parent.toolbar.clear()

		i=0
		self.typeid=IntVar()
		for type,color in Node.colors.items():
			self.parent.toolbar.add(Radiobutton(self.parent.toolbar.frame,value=i,indicatoron=0,text=type,bg=color,selectcolor=color,variable=self.typeid))
			i+=1

                self.name=Entry(self.parent.toolbar.frame)
                self.parent.toolbar.add(Label(self.parent.toolbar.frame,text="Name:",bg=self.parent.toolbar.frame.cget("bg")))
                self.parent.toolbar.add(self.name)
                self.ip=Entry(self.parent.toolbar.frame)
                self.parent.toolbar.add(Label(self.parent.toolbar.frame,text="IP:",bg=self.parent.toolbar.frame.cget("bg")))
                self.parent.toolbar.add(self.ip)

                b=Button(self.parent.toolbar.frame,text="Apply",command=self.apply)
                self.parent.toolbar.add(b)
                b.pack(side=RIGHT)
        def apply(self):
                try:
			o=self.parent.selected[0]
			o.name=string.replace(self.name.get(),",","")
			o.ip=self.ip.get()
			o.type=Node.colors.items()[self.typeid.get()][0]
			o.update(self.parent.canvas)
                except IndexError: pass
	def canvas_click(self,event):
		ModeHandler.canvas_click(self,event)
		self.name.delete(0,END)
		self.ip.delete(0,END)
		o=None
		try:
                	o=self.parent.selected[0]
         	        self.name.insert(0,o.name)
                	self.ip.insert(0,o.ip)

			i=0
			for a,b in Node.colors.items():
				if a==o.type:
					self.typeid.set(i)
					break
				i+=1
			
		
		except IndexError:
                    pass

class ModeHandler_Operations(ModeHandler):
	"""A mode handler for special node operations"""
	def massping(self):
		for id,n in self.parent.objects.items():
			n.toggle(self.parent.canvas,n.do_pingtest())
	def canvas_click(self,event):
		for id,n in self.parent.objects.items():
			n.toggle(self.parent.canvas,0)

		ModeHandler.canvas_click(self,event)
		o=None
		self.parent.toolbar.clear()
		try:
			o=self.parent.selected[0]
			self.parent.toolbar.add(Label(self.parent.toolbar.frame,text="IP: "+o.ip+"  ",bg=self.parent.toolbar.frame.cget("bg")))
			for caption,func in o.ops:
				self.parent.toolbar.add(Button(self.parent.toolbar.frame,text=caption,command=func))
		except IndexError:
			self.parent.toolbar.add(Button(self.parent.toolbar.frame,text="Mass ping",command=self.massping))
