from Tkinter import *
import os

class LinkError:
	"""Raised when trying to create an illegal link."""
	pass

class SystemNotSupported:
	"""Raised when trying to perform an illegal special op on an OS that doesn't support it."""
	pass

class Node:
	colors={"term":"gray75","rtr":"#00dbdf","hub":"#fed000","serv":"#ff8888","other":"#88ff88"}
	def __init__(self,canvas,id,x=0,y=0,name="",ip="0.0.0.0",type="term"):
                self.tag=("node","node"+str(id))
		self.id=id
		self.name=name
		self.type=type
		self.ip=ip
		self.links={}
		self.x=x
		self.y=y
		self.state=0
		self.ghost=None
		self.rect=canvas.create_rectangle(self.x-32,self.y-16,self.x+32,self.y+16,fill=self.colors[self.type],tag=self.tag)
		self.capt=canvas.create_text(self.x,self.y,text=self.name,tag=self.tag)
		self.ops=[
			("Ping", self.do_ping),
			("Telnet", self.do_telnet),
			("SSH", self.do_ssh)
			]
	def toggle(self,canvas,state=None):
		canvas.lift(self.rect)
		canvas.lift(self.capt)

		if state == None:
			self.state=self.state^1
		else:
			self.state=state

		if self.state:
			canvas.itemconfig(self.rect,fill="yellow",width=2)
		else:
			canvas.itemconfig(self.rect,fill=self.colors[self.type],width=1)
	def update(self,canvas):
		canvas.coords(self.rect,self.x-32,self.y-16,self.x+32,self.y+16)
		canvas.coords(self.capt,self.x,self.y)
		canvas.itemconfig(self.capt,text=self.name)
	def update_links(self,canvas):
		for to,h in self.links.items():
			if h!=None:
				canvas.coords(h,self.x,self.y,to.x,to.y)
			else:
				h=to.links[self]
				canvas.coords(h,to.x,to.y,self.x,self.y)
	def grab(self,canvas,x,y):
		if self.ghost: canvas.delete(self.ghost)
		self.ghost=canvas.create_rectangle(self.x-32,self.y-16,self.x+32,self.y+16,fill="gray90",outline="white")
		canvas.lower(self.ghost,ALL)
		self.grabx=x-self.x
		self.graby=y-self.y
	def drag(self,canvas,x,y):
		self.x=x-self.grabx
		self.y=y-self.graby
		self.update(canvas)
	def drop(self,canvas):
		if self.ghost: canvas.delete(self.ghost)
		self.ghost=None
		self.update_links(canvas)
	def link(self,to,canvas,color):
		if not len(color): color="black"
		if self.links.has_key(to): raise LinkError
		if self.id<to.id:
			h=canvas.create_line(self.x,self.y,to.x,to.y,width=2,tag="link",fill=color)
			canvas.lower(h)
			self.links[to]=h
		else:
			self.links[to]=None
	def unlink(self,to,canvas):
		try:
			h=self.links[to]
			if h!=None:
				canvas.delete(h)
			del self.links[to]
			to.unlink(self,canvas)
		except KeyError: pass
	def unlink_by_handle(self,handle,canvas):
		for to,h in self.links.items():
			if h==handle:
				self.unlink(to,canvas)
	def delete(self,canvas):
		for to,h in self.links.items():
			self.unlink(to,canvas)
		try:
			canvas.delete(self.ghost)
			canvas.delete(self.rect)
			canvas.delete(self.capt)
		except IndexError: pass
	# special ops
	def do_ping(self):
		if os.name=="posix":
			os.system("echo xterm -e \'ping %s\'|/bin/sh" % self.ip)
		elif os.name=="nt":
			os.system("ping %s" % self.ip)
		else: raise SystemNotSupported
	def do_pingtest(self):
		if os.name=="posix":
			return os.system("ping -c 1 %s > /dev/null 2>&1" % self.ip)==0
		else: raise SystemNotSupported
	def do_telnet(self):
		if os.name=="posix":
			os.system("echo xterm -e \'telnet %s\'|/bin/sh" % self.ip)
		elif os.name=="nt":
			os.system("telnet %s" % self.ip)
		else: raise SystemNotSupported
	def do_ssh(self):
		if os.name=="posix":
			os.system("echo xterm -e \'ssh %s\'|/bin/sh" % self.ip)
		else: raise SystemNotSupported
